 1  --
   2  -- Tedder
   3  -- Class for all Tedders
   4  -- Note: the rotor animation is not perfectly synchronized, thus do not attach any gameplay relevant stuff to the rotor animation
   5  --
   6  -- @author  Stefan Geiger
   7  -- @date  11/05/08
   8  --
   9  -- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
  10  
  11  Tedder = {};
  12  source("dataS/scripts/vehicles/specializations/SetTurnedOnEvent.lua");
  13  source("dataS/scripts/vehicles/specializations/TedderAreaEvent.lua");
  14  
  15  function Tedder.prerequisitesPresent(specializations)
  16      return true;
  17  end;
  18  
  19  function Tedder:load(xmlFile)
  20  
  21      assert(self.setIsTurnedOn == nil, "Tedder needs to be the first specialization which implements setIsTurnedOn");
  22      self.setIsTurnedOn = Tedder.setIsTurnedOn;
  23  
  24      self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);
  25      self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));
  26      if self.groundReferenceNode == nil then
  27          self.groundReferenceNode = self.components[1].node;
  28      end;
  29  
  30      self.animTime = 0;
  31  
  32      self.rotors = {};
  33  
  34      local i=0;
  35      while true do
  36          local baseName = string.format("vehicle.rotors.rotor(%d)", i);
  37          local node = {};
  38          node.index = getXMLString(xmlFile, baseName.. "#index");
  39          local direction = Utils.getNoNil(getXMLInt(xmlFile, baseName.. "#direction"), 1);
  40          node.speed = direction*Utils.getNoNil(getXMLFloat(xmlFile, baseName.. "#speed"), -0.008);
  41          if node.index == nil then
  42              break;
  43          end;
  44          node.index = Utils.indexToObject(self.components, node.index);
  45          if node.index ~= nil then
  46              table.insert(self.rotors, node);
  47          end;
  48          i = i+1;
  49      end;Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
  50  
  51      local numTedderDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.tedderDropAreas#count"), 0);
  52      if numTedderDropAreas ~= table.getn(self.cuttingAreas) then
  53          print("Warning: Number of cutting areas and drop areas should be equal");
  54      end;
  55      self.tedderDropAreas = {}
  56      for i=1, numTedderDropAreas do
  57          self.tedderDropAreas[i] = {};
  58          local areanamei = string.format("vehicle.tedderDropAreas.tedderDropArea%d", i);
  59          self.tedderDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
  60          self.tedderDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
  61          self.tedderDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
  62      end;
  63  
  64      local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
  65      for i=1, numCuttingAreas do
  66          local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
  67          self.cuttingAreas[i].foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMinLimit"), 0);
  68          self.cuttingAreas[i].foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMaxLimit"), 1);
  69          self.cuttingAreas[i].grassParticleSystemIndex = getXMLInt(xmlFile, areanamei .. "#particleSystemIndex");
  70      end;
  71  
  72      self.speedRotatingParts = {};
  73      local i=0;
  74      while true do
  75          local baseName = string.format("vehicle.speedRotatingParts.speedRotatingPart(%d)", i);
  76          local index = getXMLString(xmlFile, baseName.. "#index");
  77          if index == nil then
  78              break;
  79          end;
  80          local node = Utils.indexToObject(self.components, index);
  81          if node ~= nil then
  82              local entry = {};
  83              entry.node = node;
  84              entry.rotationSpeedScale = getXMLFloat(xmlFile, baseName.."#rotationSpeedScale");
  85              if entry.rotationSpeedScale == nil then
  86                  entry.rotationSpeedScale = 1.0/Utils.getNoNil(getXMLFloat(xmlFile, baseName.."#radius"), 1);
  87              end;
  88  
  89              entry.foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, baseName .. "#foldMinLimit"), 0);
  90              entry.foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, baseName .. "#foldMaxLimit"), 1);
  91  
  92              table.insert(self.speedRotatingParts, entry);
  93          end;
  94          i = i+1;
  95      end;
  96  
  97      self.grassParticleSystems = {};
  98      local i=0;
  99      while true doCopyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 100          local baseName = string.format("vehicle.grassParticleSystems.grassParticleSystem(%d)", i);
 101  
 102          local particleSystem = {};
 103          particleSystem.ps = {};
 104          local ps = Utils.loadParticleSystem(xmlFile, particleSystem.ps, baseName, self.components, false, nil, self.baseDirectory)
 105          if ps == nil then
 106              break;
 107          end;
 108          particleSystem.disableTime = 0;
 109          particleSystem.isEnabled = false;
 110          table.insert(self.grassParticleSystems, particleSystem);
 111          i = i+1;
 112      end;
 113  
 114      if self.isClient then
 115  
 116          local tedderSound = getXMLString(xmlFile, "vehicle.tedderSound#file");
 117          if tedderSound ~= nil and tedderSound ~= "" then
 118              tedderSound = Utils.getFilename(tedderSound, self.baseDirectory);
 119              self.tedderSound = createSample("tedderSound");
 120              self.tedderSoundEnabled = false;
 121              loadSample(self.tedderSound, tedderSound, false);
 122              self.tedderSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tedderSound#pitchOffset"), 1);
 123              self.tedderSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tedderSound#volume"), 1.0);
 124          end;
 125      end;
 126  
 127      self.isTurnedOn = false;
 128      self.wasToFast = false;
 129  
 130      self.tedderParticleSystemFlag = self:getNextDirtyFlag();
 131  end;
 132  
 133  function Tedder:delete()
 134  
 135      for k,v in pairs(self.grassParticleSystems) do
 136          Utils.deleteParticleSystem(v.ps);
 137      end;
 138  
 139      if self.tedderSound ~= nil then
 140          delete(self.tedderSound);
 141      end;
 142  
 143  end;
 144  
 145  function Tedder:readStream(streamId, connection)
 146      local isTurnedOn = streamReadBool(streamId);
 147      local animTime = streamReadFloat32(streamId);
 148      Tedder.setRotorAnimTime(self, animTime);
 149      self:setIsTurnedOn(isTurnedOn, true);Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 150  end;
 151  
 152  function Tedder:writeStream(streamId, connection)
 153      streamWriteBool(streamId, self.isTurnedOn);
 154      streamWriteFloat32(streamId, self.animTime);
 155  end;
 156  
 157  function Tedder:readUpdateStream(streamId, timestamp, connection)
 158      if connection:getIsServer() then
 159          local hasUpdate = streamReadBool(streamId);
 160          if hasUpdate then
 161              for k,v in ipairs(self.grassParticleSystems) do
 162                  local enabled = streamReadBool(streamId);
 163                  Utils.setEmittingState(v.ps, enabled);
 164              end;
 165          end;
 166      end;
 167  end;
 168  
 169  function Tedder:writeUpdateStream(streamId, connection, dirtyMask)
 170      if not connection:getIsServer() then
 171          if bitAND(dirtyMask, self.tedderParticleSystemFlag) ~= 0 then
 172              streamWriteBool(streamId, true);
 173              for k,v in ipairs(self.grassParticleSystems) do
 174                  streamWriteBool(streamId, v.isEnabled);
 175              end;
 176          else
 177              streamWriteBool(streamId, false);
 178          end;
 179      end;
 180  end;
 181  
 182  function Tedder:mouseEvent(posX, posY, isDown, isUp, button)
 183  end;
 184  
 185  function Tedder:keyEvent(unicode, sym, modifier, isDown)
 186  end;
 187  
 188  function Tedder:update(dt)
 189  
 190      if self:getIsActiveForInput() then
 191          if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
 192              --self.isTurnedOn = not self.isTurnedOn;
 193              self:setIsTurnedOn(not self.isTurnedOn);
 194          end;
 195      end;
 196  end;
 197  
 198  function Tedder:updateTick(dt)
 199      self.wasToFast = false;Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 200      if self:getIsActive() then
 201          local hasGroundContact = false;
 202          local x,y,z = getWorldTranslation(self.groundReferenceNode);
 203          local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
 204          if terrainHeight+self.groundReferenceThreshold >= y then
 205              hasGroundContact = true;
 206          end;
 207          if hasGroundContact then
 208              local foldAnimTime = self.foldAnimTime;
 209              if self.isTurnedOn then
 210                  local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;
 211                  if self.isServer then
 212                      local cuttingAreasSend = {};
 213                      if not toFast then
 214                          local numAreas = math.min(table.getn(self.tedderDropAreas), table.getn(self.cuttingAreas));
 215                          for i=1, numAreas do
 216                              local cuttingArea = self.cuttingAreas[i];
 217                              if self:getIsAreaActive(cuttingArea) then
 218                                  local x,y,z = getWorldTranslation(cuttingArea.start);
 219                                  local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
 220                                  local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
 221  
 222                                  local dropArea = self.tedderDropAreas[i];
 223                                  local dx,dy,dz = getWorldTranslation(dropArea.start);
 224                                  local dx1,dy1,dz1 = getWorldTranslation(dropArea.width);
 225                                  local dx2,dy2,dz2 = getWorldTranslation(dropArea.height);
 226  
 227                                  table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2,dx, dz, dx1, dz1, dx2, dz2, 0, i});
 228                              end;
 229                          end;
 230                          if (table.getn(cuttingAreasSend) > 0) then
 231                              local cuttingAreasSend, bitType = TedderAreaEvent.runLocally(cuttingAreasSend);
 232                              if (table.getn(cuttingAreasSend) > 0) then
 233                                  for i=1, table.getn(cuttingAreasSend) do
 234                                      local cuttingArea = self.cuttingAreas[cuttingAreasSend[i][14] ];
 235  
 236                                      if cuttingArea.grassParticleSystemIndex ~= nil then
 237                                          local ps = self.grassParticleSystems[cuttingArea.grassParticleSystemIndex+1];
 238                                          if ps ~= nil then
 239                                              ps.disableTime = self.time + 300;
 240                                              if not ps.isEnabled then
 241                                                  ps.isEnabled = true;
 242                                                  self:raiseDirtyFlags(self.tedderParticleSystemFlag);
 243                                                  if self.isClient then
 244                                                      Utils.setEmittingState(ps.ps, true);
 245                                                  end;
 246                                              end;
 247                                          end;
 248                                      end;
 249                                  end;Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 250                                  g_server:broadcastEvent(TedderAreaEvent:new(cuttingAreasSend, bitType));
 251                              end;
 252                          end;
 253                      end;
 254                  end;
 255                  self.wasToFast = toFast;
 256              end;
 257  
 258              for k,v in pairs(self.speedRotatingParts) do
 259                  if foldAnimTime == nil or (foldAnimTime <= v.foldMaxLimit and foldAnimTime >= v.foldMinLimit) then
 260                      rotate(v.node, v.rotationSpeedScale * self.lastSpeedReal * self.movingDirection * dt, 0, 0);
 261                  end;
 262              end;
 263          end;
 264          if self.isServer then
 265              for k,v in pairs(self.grassParticleSystems) do
 266                  if self.time > v.disableTime then
 267                      if v.isEnabled then
 268                          v.isEnabled = false;
 269                          self:raiseDirtyFlags(self.tedderParticleSystemFlag);
 270                          if self.isClient then
 271                              Utils.setEmittingState(v.ps, false);
 272                          end;
 273                      end;
 274                  end;
 275              end;
 276          end;
 277  
 278          if self.isTurnedOn then
 279              self.animTime = self.animTime + dt;
 280              Tedder.setRotorAnimTime(self, self.animTime);
 281  
 282              if self.isClient then
 283                  if not self.tedderSoundEnabled and self:getIsActiveForSound() then
 284                      playSample(self.tedderSound, 0, self.tedderSoundVolume, 0);
 285                      setSamplePitch(self.tedderSound, self.tedderSoundPitchOffset);
 286                      self.tedderSoundEnabled = true;
 287                  end;
 288              end;
 289          end;
 290  
 291      end;
 292  end;
 293  
 294  function Tedder:draw()
 295      if self.isClient then
 296          if self.isTurnedOn then
 297              g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
 298          else
 299              g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 300          end;
 301  
 302          if self.wasToFast then
 303              g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
 304          end;
 305      end;
 306  end;
 307  
 308  function Tedder:onDetach()
 309      if self.deactivateOnDetach then
 310          Tedder.onDeactivate(self);
 311      else
 312          Tedder.onDeactivateSounds(self)
 313      end;
 314  end;
 315  
 316  function Tedder:onLeave()
 317      if self.deactivateOnLeave then
 318          Tedder.onDeactivate(self);
 319      else
 320          Tedder.onDeactivateSounds(self)
 321      end;
 322  end;
 323  
 324  function Tedder:onDeactivate()
 325      if self.isClient then
 326          for k,v in pairs(self.grassParticleSystems) do
 327              v.isEnabled = false;
 328              Utils.setEmittingState(v.ps, false);
 329          end;
 330      end;
 331      Tedder.onDeactivateSounds(self)
 332      self.isTurnedOn = false;
 333  end;
 334  
 335  function Tedder:onDeactivateSounds()
 336      if self.tedderSoundEnabled then
 337          stopSample(self.tedderSound);
 338          self.tedderSoundEnabled = false;
 339      end;
 340  end;
 341  
 342  function Tedder:setIsTurnedOn(isTurnedOn, noEventSend)
 343      SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
 344      self.isTurnedOn = isTurnedOn;
 345  
 346      if not isTurnedOn then
 347          if self.tedderSoundEnabled then
 348              stopSample(self.tedderSound);
 349              self.tedderSoundEnabled = false;Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
 350          end;
 351      end;
 352  end;
 353  
 354  function Tedder.setRotorAnimTime(self, animTime)
 355      self.animTime = animTime;
 356      for i=1, table.getn(self.rotors) do
 357          local rotor = self.rotors[i].index;
 358          local rotorRot = self.rotors[i].speed*animTime;
 359          --rotate(rotor, 0, rotorRot, 0);
 360          setRotation(rotor, 0 , rotorRot, 0);
 361      end;
 362  end;